@@ -18,60 +18,62 @@ http://pytango.readthedocs.io
 
 from __future__ import print_function
 
-__all__ = [ 'AccessControlType', 'ApiUtil', 'ArchiveEventInfo',
-'ArchiveEventProp', 'ArgType', 'AsynCall', 'AsynReplyNotArrived', 'AttReqType',
-'Attr', 'AttrConfEventData', 'AttrData', 'AttrDataFormat', 'AttrList',
-'AttrProperty', 'AttrQuality', 'AttrReadEvent', 'AttrSerialModel',
-'AttrWriteType', 'AttrWrittenEvent', 'Attribute', 'AttributeAlarm',
-'AttributeAlarmInfo', 'AttributeConfig', 'AttributeConfig_2',
-'AttributeConfig_3', 'AttributeDimension', 'AttributeEventInfo',
-'AttributeInfo', 'AttributeInfoEx', 'AttributeInfoList', 'AttributeInfoListEx',
-'AttributeList', 'AttributeProxy', 'ChangeEventInfo', 'ChangeEventProp',
-'Pipe', 'PipeConfig', 'PipeWriteType',
-'CmdArgType', 'CmdDoneEvent', 'CommandInfo', 'CommandInfoList',
-'CommunicationFailed', 'Connection', 'ConnectionFailed',
-'ConstDevString', 'DServer', 'DataReadyEventData', 'Database', 'DbData',
-'DbDatum', 'DbDevExportInfo', 'DbDevExportInfos', 'DbDevImportInfo',
-'DbDevImportInfos', 'DbDevFullInfo', 'DbDevInfo', 'DbDevInfos', 'DbHistory',
-'DbHistoryList', 'DbServerInfo', 'DbServerData', 'DebugIt', 'DevBoolean', 'DevCommandInfo',
-'DevDouble', 'DevEncoded', 'DevError', 'DevFailed', 'DevFloat', 'DevInt',
-'DevLong', 'DevLong64', 'DevShort', 'DevSource', 'DevState', 'DevString',
-'DevUChar', 'DevULong', 'DevULong64', 'DevUShort', 'DevEnum', 'DevVarBooleanArray',
-'DevVarCharArray', 'DevVarDoubleArray', 'DevVarDoubleStringArray',
-'DevVarFloatArray', 'DevVarLong64Array', 'DevVarLongArray',
-'DevVarLongStringArray', 'DevVarShortArray', 'DevVarStringArray',
-'DevVarULong64Array', 'DevVarULongArray', 'DevVarUShortArray',
-'DevVoid', 'DeviceAttribute', 'DeviceAttributeConfig',
-'DeviceAttributeHistory', 'DeviceClass', 'DeviceData', 'DeviceDataList',
-'DeviceDataHistory', 'DeviceDataHistoryList',
-'DeviceImpl', 'DeviceInfo', 'DeviceProxy',
-'DeviceUnlocked', 'Device_2Impl', 'Device_3Impl', 'Device_4Impl', 'Device_5Impl',
-'DispLevel', 'EncodedAttribute', 'ErrSeverity', 'ErrorIt',
-'EventData', 'EventProperties', 'EventSystemFailed', 'EventType',
-'Except', 'ExtractAs', 'FMT_UNKNOWN', 'FatalIt', 'Group', 'GroupAttrReply',
-'GroupAttrReplyList', 'GroupCmdReply', 'GroupCmdReplyList',
-'GroupReply', 'GroupReplyList', 'IMAGE', 'ImageAttr', 'InfoIt',
-'KeepAliveCmdCode', 'Level', 'LockCmdCode', 'LockerInfo', 'LockerLanguage',
-'LogIt', 'LogLevel', 'LogTarget', 'Logger', 'Logging', 'MessBoxType',
-'MultiAttribute', 'MultiAttrProp', 'MultiClassAttribute', 'NamedDevFailed',
-'NamedDevFailedList', 'NonDbDevice', 'NonSupportedFeature',
-'NotAllowed', 'NumpyType', 'PeriodicEventInfo', 'PeriodicEventProp',
-'PollCmdCode', 'PollDevice',
-'PollObjType', 'READ', 'READ_WITH_WRITE', 'READ_WRITE', 'Release', 'SCALAR',
-'SPECTRUM', 'SerialModel', 'SpectrumAttr', 'StdDoubleVector',
-'StdGroupAttrReplyVector', 'StdGroupCmdReplyVector', 'StdGroupReplyVector',
-'StdLongVector', 'StdNamedDevFailedVector', 'StdStringVector', 'SubDevDiag',
-'TangoStream', 'TimeVal', 'UserDefaultAttrProp', 'UserDefaultPipeProp', 'Util',
-'WAttribute', 'WRITE', 'WarnIt', 'WrongData', 'WrongNameSyntax', '__version__',
-'__version_description__', '__version_info__', '__version_long__',
-'__version_number__', 'alarm_flags', 'asyn_req_type', 'cb_sub_model',
-'class_factory', 'class_list', 'constants', 'constructed_class',
-'cpp_class_list', 'delete_class_list', 'get_class', 'get_classes',
-'get_constructed_class', 'get_constructed_classes', 'get_cpp_class',
-'get_cpp_classes', 'is_array_type', 'is_float_type',
-'is_int_type', 'is_numerical_type', 'is_scalar_type', 'numpy_image',
-'numpy_spectrum', 'numpy_type', 'obj_2_str', 'raise_asynch_exception',
-'seqStr_2_obj', 'AutoTangoMonitor', 'AutoTangoAllowThreads']
+__all__ = [
+    'AccessControlType', 'ApiUtil', 'ArchiveEventInfo',
+    'ArchiveEventProp', 'ArgType', 'AsynCall', 'AsynReplyNotArrived', 'AttReqType',
+    'Attr', 'AttrConfEventData', 'AttrData', 'AttrDataFormat', 'AttrList',
+    'AttrProperty', 'AttrQuality', 'AttrReadEvent', 'AttrSerialModel',
+    'AttrWriteType', 'AttrWrittenEvent', 'Attribute', 'AttributeAlarm',
+    'AttributeAlarmInfo', 'AttributeConfig', 'AttributeConfig_2',
+    'AttributeConfig_3', 'AttributeDimension', 'AttributeEventInfo',
+    'AttributeInfo', 'AttributeInfoEx', 'AttributeInfoList', 'AttributeInfoListEx',
+    'AttributeList', 'AttributeProxy', 'ChangeEventInfo', 'ChangeEventProp',
+    'Pipe', 'PipeConfig', 'PipeWriteType',
+    'CmdArgType', 'CmdDoneEvent', 'CommandInfo', 'CommandInfoList',
+    'CommunicationFailed', 'Connection', 'ConnectionFailed',
+    'ConstDevString', 'DServer', 'DataReadyEventData', 'Database', 'DbData',
+    'DbDatum', 'DbDevExportInfo', 'DbDevExportInfos', 'DbDevImportInfo',
+    'DbDevImportInfos', 'DbDevFullInfo', 'DbDevInfo', 'DbDevInfos', 'DbHistory',
+    'DbHistoryList', 'DbServerInfo', 'DbServerData', 'DebugIt', 'DevBoolean', 'DevCommandInfo',
+    'DevDouble', 'DevEncoded', 'DevError', 'DevFailed', 'DevFloat', 'DevInt',
+    'DevLong', 'DevLong64', 'DevShort', 'DevSource', 'DevState', 'DevString',
+    'DevUChar', 'DevULong', 'DevULong64', 'DevUShort', 'DevEnum', 'DevVarBooleanArray',
+    'DevVarCharArray', 'DevVarDoubleArray', 'DevVarDoubleStringArray',
+    'DevVarFloatArray', 'DevVarLong64Array', 'DevVarLongArray',
+    'DevVarLongStringArray', 'DevVarShortArray', 'DevVarStringArray',
+    'DevVarULong64Array', 'DevVarULongArray', 'DevVarUShortArray',
+    'DevVoid', 'DeviceAttribute', 'DeviceAttributeConfig',
+    'DeviceAttributeHistory', 'DeviceClass', 'DeviceData', 'DeviceDataList',
+    'DeviceDataHistory', 'DeviceDataHistoryList',
+    'DeviceImpl', 'DeviceInfo', 'DeviceProxy',
+    'DeviceUnlocked', 'Device_2Impl', 'Device_3Impl', 'Device_4Impl', 'Device_5Impl',
+    'DispLevel', 'EncodedAttribute', 'ErrSeverity', 'ErrorIt',
+    'EventData', 'EventProperties', 'EventSystemFailed', 'EventType',
+    'Except', 'ExtractAs', 'FMT_UNKNOWN', 'FatalIt', 'Group', 'GroupAttrReply',
+    'GroupAttrReplyList', 'GroupCmdReply', 'GroupCmdReplyList',
+    'GroupReply', 'GroupReplyList', 'IMAGE', 'ImageAttr', 'InfoIt',
+    'KeepAliveCmdCode', 'Level', 'LockCmdCode', 'LockerInfo', 'LockerLanguage',
+    'LogIt', 'LogLevel', 'LogTarget', 'Logger', 'Logging', 'MessBoxType',
+    'MultiAttribute', 'MultiAttrProp', 'MultiClassAttribute', 'NamedDevFailed',
+    'NamedDevFailedList', 'NonDbDevice', 'NonSupportedFeature',
+    'NotAllowed', 'NumpyType', 'PeriodicEventInfo', 'PeriodicEventProp',
+    'PollCmdCode', 'PollDevice',
+    'PollObjType', 'READ', 'READ_WITH_WRITE', 'READ_WRITE', 'Release', 'SCALAR',
+    'SPECTRUM', 'SerialModel', 'SpectrumAttr', 'StdDoubleVector',
+    'StdGroupAttrReplyVector', 'StdGroupCmdReplyVector', 'StdGroupReplyVector',
+    'StdLongVector', 'StdNamedDevFailedVector', 'StdStringVector', 'SubDevDiag',
+    'TangoStream', 'TimeVal', 'UserDefaultAttrProp', 'UserDefaultPipeProp', 'Util',
+    'WAttribute', 'WRITE', 'WarnIt', 'WrongData', 'WrongNameSyntax', '__version__',
+    '__version_description__', '__version_info__', '__version_long__',
+    '__version_number__', 'alarm_flags', 'asyn_req_type', 'cb_sub_model',
+    'class_factory', 'class_list', 'constants', 'constructed_class',
+    'cpp_class_list', 'delete_class_list', 'get_class', 'get_classes',
+    'get_constructed_class', 'get_constructed_classes', 'get_cpp_class',
+    'get_cpp_classes', 'is_array_type', 'is_float_type',
+    'is_int_type', 'is_numerical_type', 'is_scalar_type', 'numpy_image',
+    'numpy_spectrum', 'numpy_type', 'obj_2_str', 'raise_asynch_exception',
+    'seqStr_2_obj', 'AutoTangoMonitor', 'AutoTangoAllowThreads',
+    'LatestDeviceImpl']
 
 __docformat__ = "restructuredtext"
 
@@ -120,7 +122,8 @@ if os.name == 'nt':
 else:
     from . import _tango
 
-from ._tango import (AccessControlType, ApiUtil, ArchiveEventInfo,
+from ._tango import (
+    AccessControlType, ApiUtil, ArchiveEventInfo,
     AsynCall, AsynReplyNotArrived, AttReqType, Attr, AttrConfEventData,
     AttrDataFormat, AttrList, AttrProperty, AttrQuality, AttrReadEvent,
     AttrSerialModel, AttrWriteType, AttrWrittenEvent, Attribute,
@@ -176,7 +179,7 @@ from .log4tango import TangoStream, LogIt, DebugIt, InfoIt, WarnIt, \
     ErrorIt, FatalIt
 from .device_server import ChangeEventProp, PeriodicEventProp, \
     ArchiveEventProp, AttributeAlarm, EventProperties, AttributeConfig, \
-    AttributeConfig_2, AttributeConfig_3, MultiAttrProp
+    AttributeConfig_2, AttributeConfig_3, MultiAttrProp, LatestDeviceImpl
 from .pipe import PipeConfig
 from .attribute_proxy import AttributeProxy, get_attribute_proxy
 from .group import Group
